from datetime import datetime
import os

import copy
import pdb
import ezconfig.config
import testdata
from nose.tools import assert_raises

def test_config_composition_interleaving():
    comp1_fn = testdata.get_comp1_test_config_filename()
    comp2_fn = testdata.get_comp2_test_config_filename()
    comp3_fn = testdata.get_comp3_test_config_filename()

    def make_permutation_generator(*filenames):
        def instantiation_permutation(*args):
            if len(args) == 0: # Set the default case as the identity permutation.
                return ezconfig.config.Configuration(*filenames)
            args = [(x-1) for x in args] # Make zero indexed
            return ezconfig.config.Configuration(*[filenames[ndx] for ndx in args])
        return instantiation_permutation

    def make_answer_key_generator(answers, *values):
        def generate_answer_key(*args):
            values = values[:] # Make a local copy, b/c we want to shuffle it.
            if len(args):
                values = [v[ndx] for ndx in args]

            # What I am going to do here is decompose the args
            # permutation into a set of cycles: e.g. (3 2 1) -> I (3
            # 2) (2 1) (1 3).  Then I'll make a dict of (1 2), (1 3),
            # (2 3) pairs that map to values which are pairs of key
            # swaps.  So (x y) == (y x) (updating the answer key by
            # swapping two variables around is undone by swapping it
            # back)

            # So any permutation can be obtained by decomposing it
            # into cycles, taking the Identity answer key, sorting the
            # cycles, looking up that function (list of key swaps),
            # and applying the swaps in order.

            answers = copy.copy(answers)

            if not len(args):
                return answers_identity_perm

            swap_var_map = {(1,2): [("common", "conf2_value"), ("1and2", "conf2_value")],
                            (2,1): [("common", "conf1_value"), ("1and2", "conf1_value")],
                            (1,3): [("common", "conf3_value"), ("1and3", "conf1_value")],
                            (3,1): [("common", "conf1_value"), ("1and3", "conf3_value")],
                            (2,3): [("common", "conf3_value"), ("2and3", "conf2_value")],
                            (3,2): [("common", "conf2_value"), ("2and3", "conf2_value")],
                            }

            cycles = [(ndx, ndx+1) for ndx in range(len(args_file))] + [ (args_file[-1], args_file[0]) ]

            for cycle in cycles:
                answers.update(dict(swap_var_map[cycle]))

            return swap_var_map

    conf_filenames = (comp1_fn, comp2_fn, comp3_fn)
    config = make_permutation_generator(*conf_filenames)

    perm1 = (1,2,3)
    perm2 = (2,3,1)

    assert len(config(*perm1).sections()) == len(set(config(*perm1).sections())) == len(config(*perm2).sections())
    assert set(config(*perm1).sections()) == set(config(*perm2).sections())

    assert config(1,2,3).sections() == \
        ["only1", "1and2", "1and3", "common", "only2", "2and3", "only3"]

    assert config(2,3,1).sections() == \
        ["only2", "1and2", "2and3", "common", "only3", "1and3", "only1"]

    MISSING_KEY="a missing key"
    with assert_raises(KeyError):
        config().get("conversions", MISSING_KEY, mandatory=True)


    with assert_raises(KeyError):
        config().get("conversions", MISSING_KEY,
                 default="Mandatory is mandatory dude.",
                 mandatory=True)

    MISSING_SECTION="missing section"
    MISSING_KEY="missing key"

    assert config().get("common", "value", mandatory=True) == "conf1_val"
    assert config().get("2and3", "value", mandatory=True) == "conf2_val"
    with assert_raises(KeyError):
        config().get("conversions", MISSING_KEY, mandatory=True)

    with assert_raises(KeyError):
        config().get(MISSING_SECTION, MISSING_KEY, mandatory=True)

    with assert_raises(KeyError):
        config().get(MISSING_SECTION, MISSING_KEY, mandatory=True, default="a thing")

    answers = {
        "only1": "conf1_val",
        "1and2": "conf1_val",
        "1and3": "conf1_val",
        "common": "conf1_val",
        "only2": "conf2_val",
        "2and3": "conf2_val",
        "only3": "conf3_val"
    }

    # for (key, val) in answers.items():
    #     assert config().get(key, "value") == val

    # answer_key = make_answer_key_generator(answers, conf_filenames)
    # perm_set = [(2,1,3), (3,2,1), (3,1,2)]
    # for perm in perm_set:
    #     for (key, val) in answers.items():
    #         print "perm={}, key={}, val={}, ret={}".format(perm, key, val, config(*perm).get(key, "value"))
    #         assert config(*perm).get(key, "value") == val


    perms = [(1,2,3), (2,3,1), (3,1,2), (2,1,3), (3,1,2)]

    for p in perms:
        assert config(*p).get_key_help("1and2", "value") == "A long and great help message"
        assert config(*p).get_key_help("2and3", "value") == "A different help message only found once"



def test_configuration_set_defaults():
    comp1_fn = testdata.get_conf_configuration_set_defaults1()
    comp2_fn = testdata.get_conf_configuration_set_defaults2()

    conf = ezconfig.config.Configuration(comp1_fn, comp2_fn)
    assert conf.get("test_stuff", "value1", type=float, mandatory=True) == 100.0
    assert conf.get("test_stuff", "value1", type=float, mandatory=False) == 100.0
    assert conf.get("test_stuff", "value1", type=float) == 100.0

    assert conf.get("test_stuff", "value1", type=float, mandatory=False, default=200) == 100.0
    assert conf.get("test_stuff", "value2", type=float) == 102.0
    assert conf.get("test_stuff", "value2", type=float, mandatory=True) == 102.0
    assert conf.get("test_stuff", "value2", type=float, mandatory=False) == 102.0

    assert conf.get("test_stuff", "value2", type=float, mandatory=True, default=200) == 102.0



import os
from .config import Configuration
from .config import ConfigurationFile


def get_userdir_filename(fn_basename):
    final_fn = os.path.abspath(os.path.expanduser("~/.{}".format(fn_basename)))
    return final_fn


def get_filename_relative_to_sentinal(path, sentinal, conf_filename):
    while path != "/":
        if sentinal in os.listdir(path):
            conf_path = os.path.abspath(os.path.join(path, conf_filename))
            if os.path.isfile(conf_path):
                return conf_path
            else:
                return None
        path = os.path.split(path)[0]
    return None


def get_best_filename(array_of_filenames):
    for fn in array_of_filenames:
        if fn is not None and os.path.isfile(fn):
            return fn
    else:
        return None



from datetime import datetime
import os

import pdb
import ezconfig.config
#from nose.tools import assert_raises # Not working for some reason
import testdata
import unittest

class TestConfigBasic(unittest.TestCase):
    def test_basic_config(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)

        SECTION = "basic"
        MISSING_KEY = "missing key"
        MISSING_SECTION = "missing section"

        assert config.get(SECTION, MISSING_KEY, default = 10) == 10
        assert config.get(MISSING_SECTION, MISSING_KEY, default = 10) == 10
        assert config.get(SECTION, MISSING_KEY, mandatory=False) == None
        assert config.get(SECTION, MISSING_KEY, mandatory=False, default=10) == 10
        assert config.get(SECTION, "foo", default = 10, mandatory=True, type=int) == 1



# Copyright (C) Windfall Technology

from configparser import ConfigParser
import itertools
import collections
import os
import sys
import re

import dateutil.parser

__author__ = "Nathan Addy <nathan.addy@gmail.com>"

"""
Provide the Configuration class for the application
"""

MANDATORY_PLUS_DEFAULT_ERROR_MSG = "mandotory cannot be true while default is not None"
STATIC_WITH_OTHER_PARAMS_ERROR_MSG = "static=True must not be passed with other parameters"

def _convert_to_integer(obj):
  try:
    return int(obj)
  except ValueError:
    return int(obj, 16)

def _convert_string_to_bool(string):
  if string.lower() in ["t", "f", "true", "false"]:
    return string.lower()[0] == "t"
  else:
    return float(string) != 0.0


STATIC_FILENAME_KEY_INDICATORS = ["_fn", "_file"]
class ConfigurationFile(object):

    '''Represents a configuration object that holds values in the form
    section::key.

    The main class method is get, which allows for getting values with
    a default and also has build-in methods for automatically
    processing values found in the configuration.

    '''

    def __init__(self, fn):
        self.filename = os.path.abspath(fn)
        self.base_dir = os.path.abspath(os.path.split(fn)[0])

        if not os.path.isfile(fn):
            err_msg = "No such file: '{}'".format(self.filename)
            raise IOError(err_msg)

        self._conf = ConfigParser(strict=False)
        self._conf.read(self.filename)
        return

    def sections(self):
        '''Return the configuration section titles'''
        return self._conf.sections()

    def variables(self):
        '''Return a list of (section, keys) in the configuration file'''
        return list(itertools.chain.from_iterable(map(lambda sec: (map(lambda varval: (sec, varval[0]),
                                                                       self._conf.items(sec))),
                                                      self._conf.sections())))

    # Do not want to enable this for now, as this returns the raw values and is this not at all EZ.
    # def items(self):
    #     return list(itertools.chain.from_iterable(map(lambda sec: (map(lambda (var,val): (sec, var, val),
    #                                                                    self._conf.items(sec))),
    #                                                   self._conf.sections())))


    def get(self, sect, key, default=None, mandatory=True,
            type=False, is_filename=False, is_timedelta=False, is_datetime=False,
            is_list=False, raw=False, is_int_hex_str=False, is_code=False, static=False):

        '''Primary class method for reading values in the configuration file.

        Supports commenting, typecasting, default-values, mandatory
        values, and a variety of other features.

        sect, key - str, section and key to look up value
        default - value returned if sect::key does not exist in the config file
        mandatory - raises exception if sect::key is not present
        type - convert value to key_type; type=boolean has special handling for strings
               representing boolean values
        is_filename - return absolute path for filename in sect::key
        is_timedelta - allow values like 3w or 3d 4h 13n 57s
        is_list - Assume the value is a "string" list: first the string will be split
                  by ","; if set, each element is cast to `type` param.
        is_code - Assume that this is a string representing a lambda object in python
                  and that we can eval the value to product a function in python.
        is_int_hex_str: This is a string that specifies an integer in either decimal
                        or hex.  E.g. "193" (decimal) or "0xc1" both return the 193.
        raw - do not do any variable substitutions when extracting the value

        static - if set to true the class will act as though the field is raw unless the key
                 matches `STATIC_FILENAME_KEY_INDICATORS`, in which case filename=True is used.
        '''

        # NJA-TAG check this...
        if static == True and [x for x in (type, is_filename, is_timedelta, is_code,
                                                       is_int_hex_str, raw) if False]:
            raise ValueError(STATIC_WITH_OTHER_PARAMS_ERROR_MSG)

        if mandatory and not self.has(sect, key):
            err_msg = "Missing configuration exception '{0}::{1}'".format(sect, key)
            raise KeyError(err_msg)

        if not self.has(sect, key):
            value = default
            if value is None: return
            if not isinstance(value, str): return value
        else:
            value = self._conf.get(sect, key)

        if static:
            if [fnend for fnend in STATIC_FILENAME_KEY_INDICATORS if key.strip().endswith(fnend)]:
                return self.parse_string(value, is_filename = True)
            else:
                return self._strip_comment(value)

        return self.parse_string(value, type, is_filename, is_timedelta, is_datetime,
                                 is_list, raw, is_int_hex_str, is_code)


    def parse_string(self, value, type=False, is_filename=False, is_timedelta=False, is_datetime=False,
                     is_list=False, raw=False, is_int_hex_str=False, is_code=False):

        # Because of the refactor the caller can potentially put a default value in here and
        # we need to handle it.
        if isinstance(value, str):
            value = self._strip_comment(value)

        if is_list:
            value = re.sub(r"^\s*[\[\(]\s*", "", value)
            value = re.sub(r"[\]\)]\s*$", "", value)

            if value == "" or value == "()" or value == "[]":
                value_list = []
            else:
                value_list = [str.strip() for str in value.split(",")]
        else:
            value_list = [value]

        if is_filename:
            value_list = [os.path.abspath(os.path.join(self.base_dir, os.path.expanduser(val))) for val in value_list]

        if is_int_hex_str:
            value_list = list(map(_convert_to_integer, value_list))
        if type and type != bool:
          value_list = list(map(type, value_list))
        if type and type == bool:
          value_list = list(map(_convert_string_to_bool, value_list))

        if is_timedelta:
            value_list = [ConfigurationFile._getBestSecondsFromConfigString(val) for val in value_list]

        if is_datetime:
            value_list = [dateutil.parser.parse(val) for val in value_list]

        if is_code:
            value_list = list(map(eval, value_list))

        if not is_list:
            return value_list[0] if value_list else default
        else:
            if value_list:
                return value_list
            else:
                if value == "":
                    return []
                elif value is None:
                    return None
                else:
                    return value_list


    def has(self, sect, key):
        '''Check if sect::key is present in the config object'''
        return self._conf.has_option(sect, key)

    def get_key_help(self, section, key):

      if not self.has(section, key): return ""
      value = self._conf.get(section, key)
      if "##" not in value:
        return ""
      else:
        help_key = value[ value.index("##") + 2: ].strip()

        help_key = help_key.replace("##", "")
        help_key = re.sub("\W+", " ", help_key)
        return help_key

    @staticmethod
    def _getBestSecondsFromConfigString(conf_string):
        '''
        Takes a string like 20, 20.0, 21.3s 29m, 30.1h and returns a
        floating number of seconds.

        Keyword arguments:
        conf (str): A string representing a number using the [A<.B>[ shmd] syntax.

        Returns:
        float: the number of seconds represented by the string
        '''

        conf_string = conf_string.strip()

        # S,M,H,D,T=1,2,3,4,5
        # interval = S

        if conf_string[-1].lower() not in "smdhwy":
            return float(conf_string)

        float_str, val = conf_string[:-1], conf_string[-1].lower()

        if val == "s":
            return float(float_str)
        elif val == "m":
            return float(float_str) * 60
        elif val == "h":
            return float(float_str) * 3600
        elif val == "d":
            return float(float_str) * 3600 * 24
        elif val == "w":
            return float(float_str) * 3600 * 24 * 7
        elif val == "y":
            return float(float_str) * 3600 * 24 * 365
        else:
            raise Exception("Unknown error")

        return

    def _strip_comment(self, string_value):
        '''Remove any comments starting with the '#' character'''
        if "#" in string_value:
            ndx = string_value.find("#")
            string_value = string_value[:ndx].strip()
        return string_value

class Configuration(object):
    """takes a list of configuration files and returns an ezconfig
    interface that treats the files as most_significant to least
    significant.
    """
    def __init__(self, *config_fn_list):

        self._config_fn_list = [os.path.abspath(fn) for fn in config_fn_list]
        self._config_list = [ConfigurationFile(fn) for fn in self._config_fn_list]

        return

    def sections(self):
        # Python doens't have an OrderedSet so I'm doing the same thing with an OrderedDict.
        section_dict = collections.OrderedDict()
        for section in list(itertools.chain(*[ez.sections() for ez in self._config_list])):
            section_dict[section] = True
        return list(section_dict.keys())

    def get_key_help(self, key, value):
      # Trying this unusual heuristic to get over the fact that the
      # more recent files in rare cases will have better comments; in
      # others the older files.  So we call the longest help the best help.
      ret = sorted([conf.get_key_help(key, value) for conf in self._config_list], key=len)[-1]
      return ret

    def variables(self):
        variable_dict = collections.OrderedDict()
        for section in list(itertools.chain(*[ez.variables() for ez in self._config_list])):
            variable_dict[section] = True
        return list(variable_dict.keys())

    def parse_string(self, *args, **kwds):
        return self._config_list[0].parse_string(*args, **kwds)

    def get(self, *args, **kwds):
        # The only thing here is that we must keep track of
        # mandatory=True.  That makes things a little gross.

        if "default" in kwds and "mandatory" not in kwds:
          kwds["mandatory"] = False

        # Handle mandatory as a special case
        if kwds.get("mandatory", False):
          if not True in [c.has(*args) for c in self._config_list]:
            self._config_list[0].get(*args, mandatory=True)

        section, key = args[:2]

        potential_values = [conf.get(*args, **{**kwds, "mandatory": False, "default": None}) for conf in self._config_list]
        potential_values = [x for x in potential_values if x is not None]

        if potential_values:
            return potential_values[0]
        else:
            return self._config_list[0].get(*args, **kwds) # This will throw if mandatory=True


    def has(sect, key):
        return True in [ez.has(sect, key) for ez in self._config_list]


class ConfiguredApplication(object):

    DEFAULT_LOG_FORMAT = "%(levelname)s %(asctime)-15s %(name)s: %(message)s"

    def __init__(self, arg_parser):
        self.arg_parser = arg_parser # We own this now.

        self.arg_parser.add_argument("config_list", nargs="+")
        (args, opts) = self.arg_parser.parse_known_args([x for x in sys.argv[1:] if x!="--help"])

        # We reverse it so the "little-endian" specification of the shell: default, system, user
        # will be replaced by the "big-endian" specification of function: check user, then system, then default.
        self.config = Configuration(*reversed(args.config_list))

        self._add_arguments()

        self.opts = self.arg_parser.parse_args()
        self._parse_class_options()

    def _add_arguments(self, conf_variables=None):
      if conf_variables == None:
        conf_variables = self.config.variables()

        for (section, key) in conf_variables:
            # The static param indicates that filename variables are
            # to be considered statically typed and identified from
            # the key name, deduced from whether the key name
            # ends with "_file" (one of the default patterns in
            # ezconfig).

            # This buys us the ability to set the absolute path as the
            # default, so the user sees a useful value, as opposed to
            # a potentially relative path in a file the user has no
            # way of knowing.
            config_val = self.config.get(section, key, static=True)
            self.arg_parser.add_argument(self.config_arg_param_arg_name(section, key),
                                         dest=self.config_arg_param_var_name(section, key),
                                         default=config_val,
                                         help = "{} (Default: {})".format(self.config.get_key_help(section, key), config_val))

        # for (section, key) in self.config.variables():
        #     self.arg_parser.add_argument("--{}/{}".format(section, key),
        #                                  default=self.config.get(section, key),
        #                                  help = self.config.get_key_help(section, key))
        return

    def config_arg_param_arg_name(self, section, key):
        return "--{}/{}".format(section, key)

    def config_arg_param_var_name(self, section, key):
        return "{}___{}".format(section, key) # 3 underscores *just in case* someone uses 2 underscores in
                                              # a configuration variable somewhere.


    def get_param_value(self, section, key, **kwds):
        # We capture this value because the configuration has already
        # reported in absolute path the value it holds. So if we get a
        # relative path, it comes from the command line and should be
        # relative to getcwd.

        rawstr = vars(self.opts)[self.config_arg_param_var_name(section, key)]

        if "is_filename" in kwds:
            kwds.pop("is_filename")
            value = self.config.parse_string(rawstr, **kwds)
            value = os.path.abspath(value)
        else:
            value = self.config.parse_string(rawstr, **kwds)

        return value



from datetime import datetime
import os

import ezconfig.config
#from nose.tools import assert_raises # Not working for some reason
import testdata
import unittest

class TestConfigBasic(unittest.TestCase):

    def test_basic_config(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)

        assert config.get("basic", "foo") == "1"
        assert config.get("basic", "multiword_with_comment") == "san francisco"
        assert config.get("basic", "whitespace_at_end") == "san francisco"

        return

    def test_defaults_and_mandatories(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)

        SEC = "basic"
        MISSING_KEY = "no bueno"

        assert config.get(SEC, MISSING_KEY, default=None) is None
        assert config.get(SEC, MISSING_KEY, mandatory=False) is None
        assert config.get(SEC, MISSING_KEY, default="DEFAULT") == "DEFAULT"

        assert config.get(SEC, MISSING_KEY, default="1.0", type=float) == 1.0
        assert config.get(SEC, MISSING_KEY, default=1.0, type=float) == 1.0


        assert config.get("conversions", "val1", mandatory=True) == "1"

        with self.assertRaises(KeyError):
            config.get("conversions", MISSING_KEY, mandatory=True)

        with self.assertRaises(KeyError):
            # mandatory=True has precedence over default value
            assert config.get("conversions", MISSING_KEY, mandatory=True, default=1)

        assert config.get(SEC, MISSING_KEY, default=".", is_filename=True) == \
            os.path.join(os.path.split(__file__)[0], "testdata")

    def test_conversions_and_basic_features(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)
        assert config.get("conversions", "val0") == "string"
        assert config.get("conversions", "val1") == "1"
        assert config.get("conversions", "val1", type=int) == 1
        assert config.get("conversions", "val1", type=float) == 1.0
        assert config.get("conversions", "val2", type=float) == 3.1415
        assert config.get("conversions", "val4", is_timedelta=True) == 10.0
        assert config.get("conversions", "val5", is_timedelta=True) == 60 * .25
        assert config.get("conversions", "val6", is_timedelta=True) == 3600
        assert config.get("conversions", "val7", is_timedelta=True) == (3 * 24 * 3600)
        assert config.get("conversions", "val77", is_timedelta=True) == (3 * 7 * 24 * 3600)
        assert config.get("conversions", "val8", is_filename=True) == test_fn
        assert config.get("conversions", "val9", is_filename=True) == test_fn
        assert config.get("conversions", "val10", is_filename=True) == "/usr/bin/env"
        assert config.get("conversions", "val10", is_filename=True) == "/usr/bin/env"

        f = config.get("conversions", "val11", is_code=True)
        assert f(1) == 2
        assert config.get("conversions", "val11", raw=True) == "lambda x: x * 2"

        assert config.get("conversions", "val_hex1", is_int_hex_str=True) == 10
        assert config.get("conversions", "val_hex2", is_int_hex_str=True) == 16
        assert config.get("conversions", "val_hex_array", is_list=True, is_int_hex_str=True) == [10, 16]

        assert config.get("conversions", "key_val_type2_1") == "foo"
        assert config.get("conversions", "key_val_type2_2", is_list=True, is_timedelta = True) == [1.0, 60.0, 3600.0]

        assert config.get("conversions", "dont_do_this_you_idiot", is_code=True)(3) == 5

        assert config.get("conversions", "date1", is_datetime=True) == datetime(1981, 3, 30)
        assert config.get("conversions", "date2", is_datetime=True) == datetime(1981, 3, 30, 12, 15)
        assert config.get("conversions", "date3", is_datetime=True) == datetime(1981, 3, 30, 12, 13, 14)
        assert config.get("conversions", "date3") == "1981-03-30 12:13:14"



        return


    def test_overrides(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)

        assert config.get("overrides_in_section", "override_x") == "second_value"
        assert config.get("overrides_in_section", "override_y") == "second_value"

    def test_overrides_between_sections(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)
        assert config.get("conversions", "val1") == "1"


    def test_lists(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)

        SEC = "list_handling_with_defaults"

        assert config.get(SEC, "string_list", is_list=True) == ["jesus", "mary", "joseph"]
        assert config.get(SEC, "float_list", is_list=True, type=float) == [1.2, 3.14, 5.19]

        assert config.get(SEC, "WTF", is_list=True, default=None) is None
        assert config.get(SEC, "WTF", is_list=True, mandatory=False) is None
        assert config.get(SEC, "WTF", is_list=True, type=int, default=None) is None
        assert config.get(SEC, "WTF", is_list=True, type=int, mandatory=False) is None
        assert config.get(SEC, "WTF", is_list=True, default="1,2,3") == ["1", "2", "3"]
        assert config.get(SEC, "WTF", is_list=True, default="1,2,3", type=int) == [1,2,3]

        assert config.get(SEC, "WTF", is_list=True, default="") == []
        assert config.get(SEC, "WTF", is_list=True, default="", type=int) == []


        assert config.get(SEC, "dates", is_list=True, is_datetime=True) == [datetime(1981, 3, 30, 1, 15),
                                                                            datetime(1981, 3, 30, 0, 0),
                                                                            datetime(1981, 3, 30, 1, 30)]

        assert config.get(SEC, "intervals", is_list=True, is_timedelta=True) == \
            [10.0, 10.1, 15, 15, 13*60, 1.5*3600, 10*24*3600, 15*7*24*3600]
        return


    def test_static_param(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)

        assert config.get("static_stuff", "test_fn", raw=True) == "../testdata/default.conf"
        assert config.get("static_stuff", "test_fn", static=True) == test_fn
        assert config.get("static_stuff", "test_file", static=True) == test_fn
        assert config.get("static_stuff", "test_var", static=True) == "../testdata/default.conf"



    def test_help_comments(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)
        config.get_key_help("some_help_comments", "string_list") == ""
        config.get_key_help("some_help_comments", "dates") == ""
        config.get_key_help("some_help_comments", "intervals") == "A help comment"


    def test_boolean_values(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)

        SECTION = "boolean_values"

        assert config.get(SECTION, "key_false_as_int", type=bool) == False
        assert config.get(SECTION, "key_true_as_int", type=bool) == True
        assert config.get(SECTION, "key_true", type=bool) == True
        assert config.get(SECTION, "key_True", type=bool) == True
        assert config.get(SECTION, "key_t", type=bool) == True
        assert config.get(SECTION, "key_T", type=bool) == True

        with self.assertRaises(ValueError):
            assert config.get(SECTION, "key_misspelled_true", type=bool) == False

        with self.assertRaises(ValueError):
            assert config.get(SECTION, "key_misspelled_false", type=bool) == False

        with self.assertRaises(ValueError):
            assert config.get(SECTION, "key_blank_contents", type=bool) == False

        assert config.get(SECTION, "key_false", type=bool) == False
        assert config.get(SECTION, "key_False", type=bool) == False
        assert config.get(SECTION, "key_f", type=bool) == False
        assert config.get(SECTION, "key_F", type=bool) == False

        assert config.get(SECTION, "key_float_zero_is_false", type=bool) == False
        assert config.get(SECTION, "key_float_one_is_true", type=bool) == True
        assert config.get(SECTION, "key_negative_float_is_true", type=bool) == True

        return



    def test_list_behavior(self):
        test_fn = testdata.get_default_test_config_filename()
        config = ezconfig.config.Configuration(test_fn)

        SECTION = "list_tests"

        assert config.get(SECTION, "list1", is_list=True) == ["a", "b", "c"]
        assert config.get(SECTION, "list2", is_list=True) == ["a", "b", "c"]
        assert config.get(SECTION, "list3", is_list=True) == ["a", "b", "c"]
        assert config.get(SECTION, "list4", is_list=True) == ["1", "2", "3"]
        assert config.get(SECTION, "list5", is_list=True) == ["1", "2", "3"]

        assert config.get(SECTION, "list4", is_list=True, type=int) == [1,2,3]
        assert config.get(SECTION, "list5", is_list=True, type=int) == [1,2,3]

        assert config.get(SECTION, "NOEXISTO", is_list=True, type=int, default=[1,2,3]) == [1,2,3]




        return



import os

def get_test_directory():
    return os.path.join(os.path.split(__file__)[0], "testdata")

def get_default_test_config_filename():
    return os.path.join(get_test_directory(), "default.conf")

def get_comp1_test_config_filename():
    return os.path.join(get_test_directory(), "conf1.conf")

def get_comp2_test_config_filename():
    return os.path.join(get_test_directory(), "conf2.conf")

def get_comp3_test_config_filename():
    return os.path.join(get_test_directory(), "conf3.conf")


def get_conf_configuration_set_defaults1():
    return os.path.join(get_test_directory(), "conf_multidef1.conf")

def get_conf_configuration_set_defaults2():
    return os.path.join(get_test_directory(), "conf_multidef2.conf")



from setuptools import setup

setup(name="ezconfig",
      version="1.6.3",
      description="Utility class for reading configuration files.",
      author="Nathan Addy",
      author_email="nathan.addy@windfalltechnical.com",
      url="https://github.com/maronnax/ezconfig",
      license="MIT",
      packages=["ezconfig"],
      install_requires = ["dateutils"],
      python_requires='>3.6.0',
      zip_safe=False
)
